@@ -28,4 +28,4 @@ typedef struct FFDisk
 const char* ffDetectDisks(FFDiskOptions* options, FFlist* disks /* list of FFDisk */);
 
 const char* ffDetectDisksImpl(FFDiskOptions* options, FFlist* disks);
-bool ffDiskMatchMountpoint(FFDiskOptions* options, const char* mountpoint);
+bool ffDiskMatchMountpoint(FFstrbuf* folders, const char* mountpoint);
