@@ -21,10 +21,16 @@
     #include <sys/ioctl.h>
 #endif
 
-#include "gpu_asahi.h"
-
-#define FF_STR_INDIR(x) #x
-#define FF_STR(x) FF_STR_INDIR(x)
+#if defined(FF_HAVE_DRM) && defined(__aarch64__)
+    // https://github.com/alyssarosenzweig/linux/blob/agx-uapi-v7/include/uapi/drm/asahi_drm.h
+    // Found in kernel-headers-6.14.4-400.asahi.fc42.aarch64
+    #if __has_include(<drm/asahi_drm.h>)
+        #include <drm/asahi_drm.h>
+    #else
+        #include "asahi_drm.h"
+    #endif
+    #define FF_HAVE_DRM_ASAHI 1
+#endif
 
 static bool pciDetectDriver(FFstrbuf* result, FFstrbuf* pciDir, FFstrbuf* buffer, FF_MAYBE_UNUSED const char* drmKey)
 {
@@ -140,6 +146,26 @@ static const char* drmDetectAmdSpecific(const FFGPUOptions* options, FFGPUResult
         gpu->frequency = (uint32_t) (gpuInfo.max_engine_clk / 1000u);
         gpu->index = FF_GPU_INDEX_UNSET;
         gpu->type = gpuInfo.ids_flags & AMDGPU_IDS_FLAGS_FUSION ? FF_GPU_TYPE_INTEGRATED : FF_GPU_TYPE_DISCRETE;
+        #define FF_VRAM_CASE(name, value) case value /* AMDGPU_VRAM_TYPE_ ## name */: ffStrbufSetStatic(&gpu->memoryType, #name); break
+        switch (gpuInfo.vram_type)
+        {
+            FF_VRAM_CASE(UNKNOWN, 0);
+            FF_VRAM_CASE(GDDR1, 1);
+            FF_VRAM_CASE(DDR2, 2);
+            FF_VRAM_CASE(GDDR3, 3);
+            FF_VRAM_CASE(GDDR4, 4);
+            FF_VRAM_CASE(GDDR5, 5);
+            FF_VRAM_CASE(HBM, 6);
+            FF_VRAM_CASE(DDR3, 7);
+            FF_VRAM_CASE(DDR4, 8);
+            FF_VRAM_CASE(GDDR6, 9);
+            FF_VRAM_CASE(DDR5, 10);
+            FF_VRAM_CASE(LPDDR4, 11);
+            FF_VRAM_CASE(LPDDR5, 12);
+            default:
+                ffStrbufAppendF(&gpu->memoryType, "Unknown (%u)", gpuInfo.vram_type);
+                break;
+        }
 
         struct amdgpu_heap_info heapInfo;
         if (ffamdgpu_query_heap_info(handle, AMDGPU_GEM_DOMAIN_VRAM, 0, &heapInfo) >= 0)
@@ -225,6 +251,11 @@ static void pciDetectAmdSpecific(const FFGPUOptions* options, FFGPUResult* gpu,
                     gpu->shared.used = value;
             }
         }
+
+        ffStrbufSubstrBefore(pciDir, pciDirLen);
+        ffStrbufAppendS(pciDir, "/gpu_busy_percent");
+        if (ffReadFileBuffer(pciDir->chars, buffer) && (value = ffStrbufToUInt(buffer, 0)))
+            gpu->coreUsage = (double) value;
     }
 }
 
@@ -320,10 +351,7 @@ static const char* drmDetectIntelSpecific(FFGPUResult* gpu, const char* drmKey,
     {
         {
             struct drm_xe_device_query query = {
-                .extensions = 0,
                 .query = DRM_XE_DEVICE_QUERY_GT_TOPOLOGY,
-                .size = 0,
-                .data = 0,
             };
             if (ioctl(fd, DRM_IOCTL_XE_DEVICE_QUERY, &query) >= 0)
             {
@@ -590,13 +618,13 @@ static const char* detectPci(const FFGPUOptions* options, FFlist* gpus, FFstrbuf
 
 #if __aarch64__
 
-FF_MAYBE_UNUSED static const char* drmDetectAsahiSpecific(FFGPUResult* gpu, const char* name, FFstrbuf* buffer, const char* drmKey)
+FF_MAYBE_UNUSED static const char* drmDetectAsahiSpecific(FFGPUResult* gpu, const char* name, FF_MAYBE_UNUSED FFstrbuf* buffer, FF_MAYBE_UNUSED const char* drmKey)
 {
     if (sscanf(name, "agx-t%lu", &gpu->deviceId) == 1)
         ffStrbufSetStatic(&gpu->name, ffCPUAppleCodeToName((uint32_t) gpu->deviceId));
     ffStrbufSetStatic(&gpu->vendor, FF_GPU_VENDOR_NAME_APPLE);
 
-    #if FF_HAVE_DRM
+    #if FF_HAVE_DRM_ASAHI
     ffStrbufSetS(buffer, "/dev/dri/");
     ffStrbufAppendS(buffer, drmKey);
     FF_AUTO_CLOSE_FD int fd = open(buffer->chars, O_RDONLY);
@@ -609,38 +637,32 @@ FF_MAYBE_UNUSED static const char* drmDetectAsahiSpecific(FFGPUResult* gpu, cons
             .size = sizeof(paramsGlobal),
         }) >= 0)
         {
-            ffStrbufAppendF(&gpu->driver, " %u", paramsGlobal.unstable_uabi_version);
+            // They removed `unstable_uabi_version` from the struct. Hopefully they won't introduce new ABI changes.
+            gpu->coreCount = (int32_t) (paramsGlobal.num_clusters_total * paramsGlobal.num_cores_per_cluster);
+            gpu->frequency = paramsGlobal.max_frequency_khz / 1000;
+            gpu->deviceId = paramsGlobal.chip_id;
 
-            // FIXME: They will introduce ABI breaking changes. Always check the latest version
-            // https://www.reddit.com/r/AsahiLinux/comments/1ei2qiv/comment/lgm0v5s/
-            if (paramsGlobal.unstable_uabi_version == DRM_ASAHI_UNSTABLE_UABI_VERSION)
+            if (!gpu->name.length)
             {
-                gpu->coreCount = (int) paramsGlobal.num_cores_total_active;
-                gpu->frequency = paramsGlobal.max_frequency_khz / 1000;
-                gpu->deviceId = paramsGlobal.chip_id;
-
-                if (!gpu->name.length)
-                {
-                    const char* variant = " Unknown";
-                    switch (paramsGlobal.gpu_variant) {
-                    case 'G':
-                        variant = "";
-                        break;
-                    case 'S':
-                        variant = " Pro";
-                        break;
-                    case 'C':
-                        variant = " Max";
-                        break;
-                    case 'D':
-                        variant = " Ultra";
-                        break;
-                    }
-                    ffStrbufSetF(&gpu->name, "Apple M%d%s (G%d%c %02X)",
-                        paramsGlobal.gpu_generation - 12, variant,
-                        paramsGlobal.gpu_generation, paramsGlobal.gpu_variant,
-                        paramsGlobal.gpu_revision + 0xA0);
+                const char* variant = " Unknown";
+                switch (paramsGlobal.gpu_variant) {
+                case 'G':
+                    variant = "";
+                    break;
+                case 'S':
+                    variant = " Pro";
+                    break;
+                case 'C':
+                    variant = " Max";
+                    break;
+                case 'D':
+                    variant = " Ultra";
+                    break;
                 }
+                ffStrbufSetF(&gpu->name, "Apple M%d%s (G%d%c %02X)",
+                    paramsGlobal.gpu_generation - 12, variant,
+                    paramsGlobal.gpu_generation, paramsGlobal.gpu_variant,
+                    paramsGlobal.gpu_revision + 0xA0);
             }
         }
     }
