@@ -156,6 +156,8 @@ static void printDisk(FFDiskOptions* options, const FFDisk* disk, uint32_t index
             FF_FORMAT_ARG(minutes, "minutes"),
             FF_FORMAT_ARG(seconds, "seconds"),
             FF_FORMAT_ARG(milliseconds, "milliseconds"),
+            FF_FORMAT_ARG(disk->mountpoint, "mountpoint"),
+            FF_FORMAT_ARG(disk->mountFrom, "mount-from"),
         }));
     }
 }
@@ -177,6 +179,12 @@ void ffPrintDisk(FFDiskOptions* options)
             if(__builtin_expect(options->folders.length == 0, 1) && (disk->type & ~options->showTypes))
                 continue;
 
+            if (options->hideFolders.length && ffDiskMatchMountpoint(&options->hideFolders, disk->mountpoint.chars))
+                continue;
+
+            if (options->hideFS.length && ffStrbufMatchSeparated(&disk->filesystem, &options->hideFS, ':'))
+                continue;
+
             printDisk(options, disk, ++index);
         }
     }
@@ -203,6 +211,18 @@ bool ffParseDiskCommandOptions(FFDiskOptions* options, const char* key, const ch
         return true;
     }
 
+    if (ffStrEqualsIgnCase(subKey, "hide-folders"))
+    {
+        ffOptionParseString(key, value, &options->hideFolders);
+        return true;
+    }
+
+    if (ffStrEqualsIgnCase(subKey, "hide-fs"))
+    {
+        ffOptionParseString(key, value, &options->hideFS);
+        return true;
+    }
+
     if (ffStrEqualsIgnCase(subKey, "show-regular"))
     {
         if (ffOptionParseBoolean(value))
@@ -291,6 +311,18 @@ void ffParseDiskJsonObject(FFDiskOptions* options, yyjson_val* module)
             continue;
         }
 
+        if (ffStrEqualsIgnCase(key, "hideFolders"))
+        {
+            ffStrbufSetS(&options->hideFolders, yyjson_get_str(val));
+            continue;
+        }
+
+        if (ffStrEqualsIgnCase(key, "hideFS"))
+        {
+            ffStrbufSetS(&options->hideFS, yyjson_get_str(val));
+            continue;
+        }
+
         if (ffStrEqualsIgnCase(key, "showExternal"))
         {
             if (yyjson_get_bool(val))
@@ -380,6 +412,12 @@ void ffGenerateDiskJsonConfig(FFDiskOptions* options, yyjson_mut_doc* doc, yyjso
     if (!ffStrbufEqual(&options->folders, &defaultOptions.folders))
         yyjson_mut_obj_add_strbuf(doc, module, "folders", &options->folders);
 
+    if (!ffStrbufEqual(&options->hideFolders, &defaultOptions.hideFolders))
+        yyjson_mut_obj_add_strbuf(doc, module, "hideFolders", &options->hideFolders);
+
+    if (!ffStrbufEqual(&options->hideFS, &defaultOptions.hideFS))
+        yyjson_mut_obj_add_strbuf(doc, module, "hideFS", &options->hideFS);
+
     if (defaultOptions.calcType != options->calcType)
         yyjson_mut_obj_add_bool(doc, module, "useAvailable", options->calcType == FF_DISK_CALC_TYPE_AVAILABLE);
 
@@ -483,6 +521,8 @@ static FFModuleBaseInfo ffModuleInfo = {
         {"Minutes after creation", "minutes"},
         {"Seconds after creation", "seconds"},
         {"Milliseconds after creation", "milliseconds"},
+        {"Mount point / drive letter", "mountpoint"},
+        {"Mount from (device path)", "mount-from"},
     }))
 };
 
@@ -492,6 +532,12 @@ void ffInitDiskOptions(FFDiskOptions* options)
     ffOptionInitModuleArg(&options->moduleArgs, "");
 
     ffStrbufInit(&options->folders);
+    #if _WIN32 || __APPLE__ || __ANDROID__
+    ffStrbufInit(&options->hideFolders);
+    #else
+    ffStrbufInitStatic(&options->hideFolders, "/efi:/boot:/boot/efi");
+    #endif
+    ffStrbufInit(&options->hideFS);
     options->showTypes = FF_DISK_VOLUME_TYPE_REGULAR_BIT | FF_DISK_VOLUME_TYPE_EXTERNAL_BIT | FF_DISK_VOLUME_TYPE_READONLY_BIT;
     options->calcType = FF_DISK_CALC_TYPE_FREE;
     options->percent = (FFPercentageModuleConfig) { 50, 80, 0 };
@@ -500,4 +546,7 @@ void ffInitDiskOptions(FFDiskOptions* options)
 void ffDestroyDiskOptions(FFDiskOptions* options)
 {
     ffOptionDestroyModuleArg(&options->moduleArgs);
+    ffStrbufDestroy(&options->folders);
+    ffStrbufDestroy(&options->hideFolders);
+    ffStrbufDestroy(&options->hideFS);
 }
